;*********************************************************************
; Support an Intel flash with the CFI interface.  This code requires a
; flash that supports the CFI interface, but it uses only a subset of
; the interface.  This code lets the caller know if it encounters a
; physical flash part that has erase or burn requirements it doesn't
; support.
;
; The CFI subset supported is:
;   * x8 device operating in 8-bit mode
;   * Individual Block Erases via CFI command 0x20
;   * Single Byte Writes via CFI command 0x40
;   * Minimum erase block size is 4K
;
; The minimum erase block size of 4K is enforced because MachZ memory
; mapped windows must be on a 4K address boundary.
;
; Note:  The Intel flash on the EPC board was wired incorrectly.  The
;        flash is 16-bit addressable, but the address lines were wired
;        so that CPU ADDR0 is wired to the least significant chip
;        address line.  The least significant chip address line should
;        have been wired to CPU ADDR1.  The net effect is that, for
;        every word on the flash, only the least significant byte is
;        populated with data.
;
;        This code compensates for the miswiring.  Look for comments
;        such as "** Compensate for addr mistake".  When the EPC board
;        is re-wired, the code compensating for the wiring will need
;        to be removed.
;*********************************************************************
; Local params and values
blockEraseTable         dd 256 dup (0) ; Table of erase block region data
blockCount              dd ?           ; Current block number in region
blocksInRegion          dd ?           ; Count of blocks in current region
blockSize               dd ?           ; Size of current erase/lock block
deviceSize              dd ?           ; Size of flash device
devicePage              dd ?           ; Current page into memory mapped window
maxEraseTime            dd ?           ; Timeout in microseconds
maxWriteTime            dd ?           ; Timeout in microseconds

blockBegOffset          dw ?           ; Offset of block we're writing to
blockEraseRegionCnt     dw ?           ; Count of block regions to erase
currBlockOffset         dw ?           ; Offset of next data in the block
endOfBlock              dw 0           ; Indicates last byte of block written
endOfPkt                dw 0           ; Indicates last byte of packet written
nextRegionPtr           dw ?           ; Pointer to next blockEraseTable dword
regionCount             dw ?           ; Count block regions


fb_failed_handshake_msg db C_ESC, "Failed handshake during byte write at address: ", 0
fb_failed_readback_msg  db C_ESC, "Failed readback during byte write at address: " , 0


ZFL_CTL_PORT  EQU 218H
ZFL_DATB_PORT EQU 219H
ZFL_DATW_PORT EQU 21AH
;WINDOWBASE    EQU 0E000H              ; 64K BIOS
WINDOWBASE    EQU 0C000H               ; 256K BIOS
WINDOWSIZE    EQU 10000H

;*********************************************************************
; flash_banner
; Print an opening banner for this flash burner
;
IntelMsg db "Intel StrataFlash BUR Programmer", C_CR, C_LF, 0

flash_banner proc near
         push es
         push cs
         pop es
         mov di, offset IntelMsg
         xor cx,cx
         call SerSend
         call SerRecWait
         call SerRecWait
         pop es
         ret
flash_banner endp

;*********************************************************************
; flash_confirm
; Procedure to confirm the flash is consistent with the programmer
; code embedded herein. Return is zero if the burner cannot burn the
; flash it found. Return is zero if good to erase and burn.
;
; Proc should print a flash specific message if inconsistent
;
;  return:  ax=0: flash good to go
;           ax=non-zero: flash not consistent with programmer
;

; Local messages
fc_query_error db C_ESC, "CFI query should have returned QRY, returned: ", 0

flash_confirm proc near
         push  ebx
         push  ecx
         push  di
         push  si
         push  ds
         push  es

         ; ds points to the flash window
         push WINDOWBASE
         pop  ds

         ; Configure mem_cs0, segement E000, 64Kbyte Window, first page
         WRITE_ZFL32 26H, WINDOWBASE*16      ; Map it to e0000
         WRITE_ZFL32 2AH, WINDOWSIZE-1       ; 64K window
         xor   eax, eax
         mov   dword ptr cs:[devicePage], eax; Save the page value
         WRITE_ZFL32 2EH, eax                ; First page

         ; Configure cs0 for read/write, 8 bit mode
         READ_ZFL08  5AH
         and   al, 0EEH         ; CS0 now read/write access
         or    al, 001H         ; CS0 now 8 bit access
         WRITE_ZFL08 5AH, al

         ; Confirm CFI interface - reused from ZF Microsytems source code
         xor   si, si
         mov   byte ptr ds:[si+055H], 98h ; CFI "Read Query" command
         mov   eax, ds:[si+10h]           ; get data
         and   eax, 00FFFFFFH             ; mask last byte - it's not in 'QRY'
         cmp   eax, 00595251H             ; look for 'QRY' "little-endian" style
         jz    fc_gotit

         ; Wrong answer. Pump a message with the readback data to serial port
fc_errexit:
         mov   di, offset fc_query_error
         push  cs
         pop   es
         xor   cx, cx
         call  SerSend
         call  SerOut32
         mov   ax, 1
         pop   es
         pop   ds
         pop   si
         pop   di
         pop   ecx
         pop   ebx
         ret

fc_gotit:
         ; Get the size of the flash device
         mov   cl, byte ptr ds:[si+027H]     ; CFI Device Size
         xor   eax, eax                      ; Prepare for multiply
         mov   eax, 1                        ; Get a 1 in eax
         shl   eax, cl                       ; The form is 2**n bytes
         shr   eax, 1                        ; ** Compensate for addr mistake
         mov   dword ptr cs:[deviceSize], eax; Save it

         ; Read the erase block data
         xor   ax, ax
         mov   al, byte ptr ds:[si+02CH]     ; CFI Number of erase block regions
         mov   cs:[regionCount], ax          ; Save it for later reference
         mov   cs:[blockEraseRegionCnt], ax  ; Use this one as the running count
         push  si                            ; Save the source index
         push  di                            ; Save the destination index
         mov   di, offset blockEraseTable    ; Get our destination table offset
fc_EB:
         mov   ecx, dword ptr ds:[si+02DH]   ; Get the next flash table entry
         mov   dword ptr cs:[di], ecx        ; Put it in our own table
         add   si, 4                         ; Point to the next table entries
         add   di, 4
         dec   al                            ; done?
         jnz   fc_EB
         pop   di                            ; Restore the destination index
         pop   si                            ; Restore the source index

         ; Figure out the block erase timeout
         mov   cl, byte ptr ds:[si+021H]     ; CFI Typical block erase timeout
         xor   eax, eax                      ; Prepare for multiply
         mov   ax, 1                         ; Get a 1 in ax
         shl   ax, cl                        ; The form is 2**n milliseconds
         mov   cl, byte ptr ds:[si+025H]     ; CFI Maximum block erase timeout
         xor   ebx, ebx                      ; Prepare for multiply
         mov   bx, 1                         ; Get a 1 in bx
         shl   bx, cl                        ; The form is 2**n (no units)
         mul   ebx                           ; EAX gets the maximum timeout (ms)
         mov   ebx, 1000                     ; Make it microseconds
         mul   ebx
         mov   dword ptr cs:[maxEraseTime], eax

         ; Figure out the byte write timeout
         mov   cl, byte ptr ds:[si+01FH]     ; CFI Typical byte write timeout
         xor   eax, eax                      ; Prepare for multiply
         mov   ax, 1                         ; Get a 1 in ax
         shl   ax, cl                        ; The form is 2**n microseconds
         mov   cl, byte ptr ds:[si+023H]     ; CFI Maximum byte write timeout
         xor   ebx, ebx                      ; Prepare for multiply
         mov   bx, 1                         ; Get a 1 in bx
         shl   bx, cl                        ; The form is 2**n (no units)
         mul   ebx                           ; EAX gets the maximum timeout (us)
         mov   dword ptr cs:[maxWriteTime], eax
         xor   si, si                        ; Put device in read array mode
         mov   byte ptr ds:[si], 0FFH
         xor   eax, eax                      ; Good return value
         pop   es
         pop   ds
         pop   si
         pop   di
         pop   ecx
         pop   ebx
         ret
flash_confirm endp

;*********************************************************************
; flash_parseRegion
; Procedure to parse the region information from the stored
; blockEraseTable.
;
;  intputs: cs:[si] points to the dword of region info in the
;              blockEraseTable to parse.
;
;  outputs: blockSize      - dword local memory location
;           blocksInRegion - dword local memory location
;
;  return:  ax = 0: success
;           ax = non-zero: failed -- block size is smaller than 4K
;           si = offset to next region info dword
;
flash_parseRegion proc near
         push  ebx
         push  ecx
         mov   ecx, dword ptr cs:[si]           ; Get the region info
         add   si, 4                            ; Increment for next time around
         mov   ebx, ecx
         and   ebx, 0FFFF0000H                  ; Get the block size
         jz    fp_128                           ; Handle 128-byte case
         shr   ebx, 8                           ; Put into units of bytes
         shr   ebx, 1                           ; ** Compensate for addr mistake
         jmp   fp_cont
fp_128:  mov   ebx, 128                         ; 0 means 128-byte block
         shr   ebx, 1                           ; ** Compensate for addr mistake
fp_cont: cmp   ebx, 01000H                      ; Enforce 4K min block size
         jae   fp_good
         mov   ax, 1                            ; Error return value
         jmp   fp_done

fp_good: mov   dword ptr cs:[blockSize], ebx    ; Save the block size
         and   ecx, 0000FFFFH                   ; Get the block count
         inc   ecx                              ; Per CFI spec.
         mov   dword ptr cs:[blocksInRegion], ecx; Save the block count
         xor   ax, ax                           ; Good return value

fp_done: pop   ecx
         pop   ebx
         ret
flash_parseRegion endp

;*********************************************************************
; flash_erase
; Procedure to erase the entire flash block by block.
;
;  return:  ax=0: success
;           ax=non-zero: failed
;
;  register usage:
;     eax - general purpose
;     ebx - size (in bytes) of current block to erase
;     ecx - count of blocks to erase in current region
;     edx - current total of bytes left to erase in the device
;     di  - offset of block address on the device
;     si  - offset of blockEraseTable in local storage
;     ds  - flash data segment
;     es  - code segment for error messages
;
fe_diag_msg    db C_ESC, "Intermediate value: ", 0
fe_erase_fail  db C_ESC, "Flash block erase failure at address: ", 0
fe_lock_fail   db C_ESC, "Flash block lock failure at address: ", 0
fe_smblk_fail  db C_ESC, "Flash block smaller than 4K at address: ", 0
fe_unlk_fail   db C_ESC, "Flash block unlock failure at address: ", 0
fe_blk_erased  db "E", 0

flash_erase_segment proc near
         push  ebx
         push  ecx
         push  edx
         push  di
         push  si
         push  ds
         push  es

         ; ds points to the flash window
         push WINDOWBASE
         pop  ds

         ; Set up block region count, block count, block size, etc.
         mov   edx, dword ptr cs:[deviceSize]   ; Get the device size
         xor   di, di                           ; Initialize destination index
         mov   si, offset blockEraseTable       ; Initialize source index

fe_newRegion:
         call  flash_parseRegion                ; si gets incremented by 4
         cmp   ax, 0                            ; Block size too small?
         je    fe_goodBlock
         mov   eax, dword ptr cs:[devicePage]   ; Display error message
         mov   di, offset fe_smblk_fail
         jmp   fe_diagexit                      ; And exit

fe_goodBlock:
         mov   ebx, dword ptr cs:[blockSize]       ; Get the block size
         mov   ecx, dword ptr cs:[blocksInRegion]  ; Get the block count
   cmp ebx, 01000H
   jz  fe_newBlock
;  sub ecx, 7
   sub ecx, 8

         ; Reset the lock bit for a block.
fe_newBlock:
   mov eax, dword ptr cs:[devicePage]
;  cmp eax, 039000H
;  jb  fe_next
;  mov eax, ebx
;  mov di, offset fe_diag_msg
;  jmp fe_diagexit
fe_next:
         mov   byte ptr ds:[di], 060H           ; Request CFI lock bit set/rst
         mov   byte ptr ds:[di], 0D0H           ; Request CFI lock bit reset
fe_waitUnlock:
         mov   al, byte ptr ds:[di]             ; Read the status register
         test  al, 080H                         ; MSB indicates complete
         jz    fe_waitUnlock                    ; Wait until unlock is finished
         cmp   al, 080H                         ; Successful?
         je    fe_erase                         ; Yes
         mov   eax, dword ptr cs:[devicePage]   ; Else display error message
         mov   di, offset fe_unlk_fail
         jmp   fe_diagexit                      ; And quit

         ; Erase a block.
fe_erase:
         mov   byte ptr ds:[di], 050H           ; Request CFI clear status reg
         mov   byte ptr ds:[di], 020H           ; Request CFI block erase
         mov   byte ptr ds:[di], 0D0H           ; Request CFI confirm
fe_waitErase:
         mov   al, byte ptr ds:[di]             ; Read the status register
         test  al, 080H
         jz    fe_waitErase                     ; Wait until erase is finished
         cmp   al, 080H                         ; Successful?
         je    fe_cleanBlock                    ; Yes
         mov   eax, dword ptr cs:[devicePage]   ; Else display error message
         mov   di, offset fe_erase_fail
         jmp   fe_diagexit                      ; And quit

         ; Clean up block after erase
fe_cleanBlock:
         mov   byte ptr ds:[di], 050H           ; Request CFI clear status reg
         mov   byte ptr ds:[di], 060H           ; Request CFI lock bit set/clear
         mov   byte ptr ds:[di], 001H           ; Request CFI lock bit set
fe_waitLock:
         mov   al, byte ptr ds:[di]             ; Read the status register
         test  al, 080H                         ; MSB indicates complete
         jz    fe_waitLock                      ; Wait until lock is finished
         cmp   al, 080H                         ; Successful?
         je    fe_nextBlock                     ; Yes
         mov   eax, dword ptr cs:[devicePage]   ; Else display error message
         mov   di, offset fe_lock_fail
         jmp   fe_diagexit                      ; And quit

         ; Set up for next block
fe_nextBlock:
         mov   byte ptr ds:[di], 050H           ; Request CFI clear status reg
         push  di
         push  ecx
         mov   di, offset fe_blk_erased         ; Indicate to client we erased
         push  cs                               ;   a block
         pop   es
         xor   cx, cx
         call  SerSend
         pop   ecx
         pop   di
         sub   edx, ebx                         ; Done?
         jbe   fe_done
         mov   eax, dword ptr cs:[devicePage]   ; Get the last page value
         add   eax, ebx                         ; Increment to next block
         mov   dword ptr cs:[devicePage], eax   ; Put it back
         push  edx                              ; Save the total byte count
         WRITE_ZFL32 2EH, eax                   ; Point page to next block
         pop   edx                              ; Restore the count
         dec   ecx                              ; Last block of region?
         jnz   fe_newBlock                      ; Another block, same region

         ; Set up for next region
         mov   ax, cs:[blockEraseRegionCnt]     ; Get block region count
         dec   ax
         jz    fe_done                          ; None left?
         mov   cs:[blockEraseRegionCnt], ax     ; Put it back
         jmp   fe_newRegion                     ; New region

fe_done:
         ; Part is now erased
         mov   byte ptr ds:[0], 0FFH            ; Reset flash to read array mode
         xor   eax, eax                         ; Good return value
;        mov   eax, 010000H
         mov   dword ptr cs:[devicePage], eax   ;Prepare for writing data
         WRITE_ZFL32 2EH, eax                   ; Point back to window page 0
         mov   eax, 9
         mov   dword ptr cs:[blockCount], eax
         xor   eax, eax
         mov   cs:[blockBegOffset], ax
         mov   cs:[currBlockOffset], ax
         mov   si, offset blockEraseTable
         call  flash_parseRegion                ; si gets incremented by 4
         mov   cs:[nextRegionPtr], si           ; Point to the next block data
; mov di, offset fe_diag_msg
; jmp fe_diagexit
         pop   es
         pop   ds
         pop   si
         pop   di
         pop   edx
         pop   ecx
         pop   ebx
         ret

         ; Pump a message with the readback data to serial port
fe_diagexit:
         mov   byte ptr ds:[0], 050H            ; Clear status register
         mov   byte ptr ds:[0], 0FFH            ; Reset to read array mode
         push  cs
         pop   es
         xor   cx, cx
         call  SerSend
         call  SerOut32
         mov   ax, 1
         pop   es
         pop   ds
         pop   si
         pop   di
         pop   edx
         pop   ecx
         pop   ebx
         ret
flash_erase_segment endp

;*********************************************************************
; flash_burn_pkt
; Procedure to burn bytes from a packet to the flash.
;
;  return:  ax=0: success
;           ax=non-zero: failed
;
;     cx - count of data in the packet
;     di - offset of data to write to the flash through the memory
;           window
;     si - offset of data in the packet
;
fb_diag_msg    db C_ESC, "Intermediate value: ", 0
fb_read_fail   db C_ESC, "Flash readback failure at address: ", 0
fb_write_fail  db C_ESC, "Flash write failure at address: ", 0
fb_lock_fail   db C_ESC, "Flash block lock failure at address: ", 0
fb_smblk_fail  db C_ESC, "Flash block smaller than 4K at address: ", 0
fb_unlk_fail   db C_ESC, "Flash block unlock failure at address: ", 0
fb_end_fail    db C_ESC, "Flash failure - end of device at address: ", 0
fb_blk_written db "W", 0

flash_burn_pkt proc near
         push  ebx
         push  ecx
         push  edx
         push  di
         push  si
         push  ds
         push  es

         ; Check if we should be done
         mov   eax, dword ptr cs:[devicePage]
         mov   ebx, dword ptr cs:[deviceSize]
         cmp   eax, ebx
         jbe   fb_begin
         push  cs
         pop   es
         mov   di, offset fb_end_fail
         jmp   fb_diagexit

         ; Establish the flash segment
fb_begin:
         push  WINDOWBASE
         pop   ds

         ; Establish the data pointer and count from the packet
         mov   cx, PKT_SIZE - 3
         mov   si, offset dbuf + 2

         ; New packet, new or existing flash block
         xor   ax, ax
         mov   cs:[endOfBlock], ax
         mov   cs:[endOfPkt], ax

         ; Reset the lock bit for the block.
fb_newBlock:
         mov   di, cs:[blockBegOffset]          ; Beginning address of block
         mov   byte ptr ds:[di], 060H           ; Request CFI lock bit set/rst
         mov   byte ptr ds:[di], 0D0H           ; Request CFI lock bit reset
fb_waitUnlock:
         mov   al, byte ptr ds:[di]             ; Read the status register
         test  al, 080H                         ; MSB indicates complete
         jz    fb_waitUnlock                    ; Wait until unlock is finished
         cmp   al, 080H                         ; Successful?
         je    fb_prepWrite                     ; Yes
         xor   eax, eax
         mov   ax, cs:[blockBegOffset]          ; Else display error message
         mov   di, offset fb_unlk_fail
         jmp   fb_diagexit                      ; And quit

         ; Set up for first byte to write
fb_prepWrite:
         mov   byte ptr ds:[di], 050H           ; Request CFI clear status reg
         mov   di, cs:[currBlockOffset]         ; Pick up where we left off

         ; Write a byte
fb_write:
         mov   byte ptr ds:[di], 040H           ; Request CFI write byte
         mov   al, byte ptr cs:[si]             ; Retrieve byte from packet
         mov   bl, al                           ; Save byte for readback
         mov   byte ptr ds:[di], al             ; Write the data to the flash
fb_waitWrite:
         mov   al, byte ptr ds:[di]             ; Read the status register
         test  al, 080H
         jz    fb_waitWrite                     ; Wait until write is finished
         cmp   al, 080H                         ; Successful?
         je    fb_readBack                      ; Yes
;  xor ecx, ecx
;  mov cl, al
;  shl ecx, 24
;  mov   eax, dword ptr cs:[blocksInRegion]
;  mov   eax, dword ptr cs:[blockCount]
   mov   eax, dword ptr cs:[blockSize]
;        mov   eax, dword ptr cs:[devicePage]
;        xor   ebx, ebx
;        mov   bx, di                           ; Else display error message
;        add   eax, ebx
;  add eax, ecx
         mov   di, offset fb_write_fail
         jmp   fb_diagexit                      ; And quit

         ; Perform readback test
fb_readBack:
         mov   byte ptr ds:[di], 050H           ; Request CFI clear status reg
         mov   byte ptr ds:[di], 0FFH           ; Request CFI read array mode
         cmp   bl, byte ptr ds:[di]             ; Readback successful?
         je    fb_incPtrs                       ; Yes
         mov   eax, dword ptr cs:[devicePage]
         xor   ebx, ebx                         ; Else display error message
         mov   bx, di
         add   eax, ebx
         mov   di, offset fb_read_fail
         jmp   fb_diagexit                      ; And quit

fb_incPtrs:
         xor   eax, eax                         ; Increment and save the
         mov   ax, di                           ;  pointer to flash
         inc   di
         inc   eax
         mov   cs:[currBlockOffset], di
         cmp   eax, dword ptr cs:[blockSize]    ; Last byte of flash block?
         jb    fb_testPkt                       ; Nope
         mov   word ptr cs:[endOfBlock], 1      ; Yes, make a note of it

fb_testPkt:
         inc   si                               ; Increment packet byte
         dec   cx                               ; Done with the packet?
         jnz   fb_testLock                      ; Nope
         mov   word ptr cs:[endOfPkt], 1        ; Yes, make a note of it

fb_testLock:
         cmp   word ptr cs:[endOfBlock], 0      ; Done with current block?
         jnz   fb_lockBlock                     ; Yes, lock it
         cmp   word ptr cs:[endOfPkt], 0        ; Done with packet?
         jnz   fb_lockBlock                     ; Yes, lock the block we're on
         jmp   fb_write                         ; No, continue writing from pkt

fb_lockBlock:
         mov   di, cs:[blockBegOffset]          ; Block beginning address
         mov   byte ptr ds:[di], 060H           ; Request CFI lock bit set/clr
         mov   byte ptr ds:[di], 001H           ; Request CFI lock bit set
fb_waitLock:
         mov   al, byte ptr ds:[di]             ; Read the status register
         test  al, 080H                         ; MSB indicates complete
         jz    fb_waitLock                      ; Wait until lock is finished
         cmp   al, 080H                         ; Successful?
         jz    fb_testEnd                       ; Yes
         xor   eax, eax
         mov   ax, cs:[blockBegOffset]          ; Else display error message
         mov   di, offset fb_lock_fail
         jmp   fb_diagexit                      ; And quit

fb_testEnd:
         mov   byte ptr ds:[di], 050H           ; Request CFI clear status reg
         cmp   word ptr cs:[endOfBlock], 0      ; Done with block?
         jnz   fb_endBlock                      ; Yes, go to the next block
                                                ;  before we exit
         jmp   fb_done                          ; No, just exit

         ; Set up for the new block
fb_endBlock:
         xor   ax, ax                           ; Reset block state
         mov   cs:[blockBegOffset], ax
         mov   cs:[currBlockOffset], ax
         mov   eax, dword ptr cs:[devicePage]   ; Increment to the next page
         mov   ebx, dword ptr cs:[blockSize]
         add   eax, ebx
;  mov   di, offset fb_diag_msg
;  jmp   fb_diagexit
         mov   dword ptr cs:[devicePage], eax   ; Store the new page
         WRITE_ZFL32 2EH, eax                   ; Point page to next block
         mov   eax, dword ptr cs:[blockCount]   ; Last block of region?
         inc   eax
         mov   ebx, dword ptr cs:[blocksInRegion]
         cmp   eax, ebx
         jae   fb_newRegion                     ; Yes, go set up the region
         mov   dword ptr cs:[blockCount], eax   ; Put the new block count back
         cmp   word ptr cs:[endOfPkt], 0        ; Done with packet also?
         jne   fb_done
         jmp   fb_newBlock                      ; Else new block, same region

fb_newRegion:
;  mov   eax, dword ptr cs:[devicePage]
;  mov   di, offset fb_diag_msg
;  jmp   fb_diagexit
         push  si                               ; Save the packet pointer
         mov   si, cs:[nextRegionPtr]           ; Last region?
         mov   ax, cs:[regionCount]
         shl   ax, 2                            ; Multiply by 4 bytes per dword
         add   ax, offset blockEraseTable       ; Point into the region table
         cmp   si, ax                           ; Done?
         pop   si                               ; Normalize the stack
         jae   fb_done
         push  si                               ; Save the packet pointer
         mov   si, cs:[nextRegionPtr]           ; Last region?
         call  flash_parseRegion                ; si gets incremented by 4
         cmp   ax, 0                            ; Block size too small?
         je    fb_goodBlock
         pop   si                               ; Normalize the stack
         mov   eax, dword ptr cs:[devicePage]   ; Display error message
         mov   di, offset fb_smblk_fail
         jmp   fb_diagexit                      ; And exit

fb_goodBlock:
;  mov   eax, dword ptr cs:[blockSize]
;  mov   di, offset fb_diag_msg
;  jmp   fb_diagexit
         mov   cs:[nextRegionPtr], si           ; Put the region pointer back
         pop   si                               ; Get back the packet pointer
         cmp   word ptr cs:[endOfPkt], 0        ; Done with packet also?
         jnz   fb_done
         jmp   fb_newBlock                      ; Else handle the next block

fb_done:
         ; Packet is now written
         xor   eax, eax                         ; Good return value
         pop   es                               ; Restore registers
         pop   ds
         pop   si
         pop   di
         pop   edx
         pop   ecx
         pop   ebx
         ret

         ; Pump a message with the readback data to serial port
fb_diagexit:
;        mov   byte ptr ds:[0], 050H            ; Clear status register
         mov   byte ptr ds:[0], 0FFH            ; Reset to read array mode
         push  cs
         pop   es
         xor   cx, cx
         call  SerSend
         call  SerOut32
         mov   ax, 1
         pop   es                               ; Restore registers
         pop   ds
         pop   si
         pop   di
         pop   edx
         pop   ecx
         pop   ebx
         ret
flash_burn_pkt endp

;*********************************************************************
; flash_burn_pkt
;
;  return:  ax=0: success
;           ax=non-zero: failed
;
flash_burn_pkt_old proc near
         ; save regs
         push bx
         push cx
         push ds
         push es
         push si
         push di

         ; setup regs for accessing the pkt buffer and region of flash
         push cs
         pop  ds
         push WINDOWBASE
         pop  es
         mov  cx, PKT_SIZE - 3
         mov  di, cs:[currBlockOffset]
         mov  si, offset dbuf + 2

fb_cx_lp:
         mov   byte ptr es:[di], 40h      ; Write Command
         mov   al, byte ptr ds:[si]       ; Retrieve byte from pkt buf
         mov   byte ptr es:[di], al       ; Give it the data
         mov   bl, al
         push  cx
         mov   cx, 500
fb_WaitWriting:
         cmp   byte ptr es:[di], 80h      ; see if "ready" bit is set
         jz    fb_ByteOk                  ; jump if done
         in    al, 80h                    ; create one ISA cycle for delay
         dec   cx                         ; not done. dec counter
         jnz   fb_WaitWriting             ; jump if counter not expired

         ; Phooey. Handshake did not occur within spec time. Send a
         ; message and return an error code.
         xor cx, cx
         mov di, offset fb_failed_handshake_msg
         push cs
         pop  es
         call SerSend
         mov ax, di
         call SerOut16
         call CRLF
         mov  ax, 1
         jmp  short fb_exit

fb_ByteOk:
         mov   byte ptr es:[di], 0FFh     ; go to array mode for 8-bit device
         cmp   bl, es:[di]
         jz    fb_next_byte

         ; Phooey. Readback failed.
         xor cx, cx
         mov di, offset fb_failed_readback_msg
         push cs
         pop  es
         call SerSend
         mov ax, di
         call SerOut16
         call CRLF
         mov  ax, 2
         jmp  short fb_exit

fb_next_byte:
         inc si
         inc di
         dec cx
         jnz fb_cx_lp

         ; Pkt is now completely written. Save next address for next time
         mov cs:[currBlockOffset], di
         ; Indicate no error
         xor ax, ax

fb_exit:
         ; We are outa here. Restore regs for caller
         pop di
         pop si
         pop es
         pop ds
         pop cx
         pop bx
         ret
flash_burn_pkt_old endp

